BagMemory = class( Turbine.UI.Window );

function BagMemory:Constructor()
	Turbine.UI.Window.Constructor( self );
	self:InitDefaults();
	self:InitWindow();
	self:LoadSettings();
	self:InitBackpack();
	self:SetVisible(true);
end
function BagMemory:Destroy()
    self:SetVisible(false);
    self:SetWantsUpdates(false);
end
function BagMemory:Update(args)
end
function BagMemory:BackPackRefresh(args)
	local item;
	if args==nil then
		for i=1,BackPack:GetSize(),1 do
			item=BackPack:GetItem(i);
			if item~=nil then
				self.Settings.Items[item:GetName()]=i;
				--Turbine.Shell.WriteLine("item "..item:GetName().." saved on position "..i);
			end
		end
		self:SaveSettings();
	else
		if args.NewIndex~=nil and args.Item~=nil then
			self.Settings.Items[args.Item:GetName()]=args.NewIndex;
			self:SaveSettings();
			--Turbine.Shell.WriteLine("item "..args.Item:GetName().." saved on position "..args.NewIndex);
		end
		if args.Index~=nil and args.Item~=nil then
			self.Settings.Items[args.Item:GetName()]=args.Index;
			self:SaveSettings();
			--Turbine.Shell.WriteLine("item "..args.Item:GetName().." saved on position "..args.Index);
		end
	end
end
function BagMemory:BackPackMemory(args)
	local targetIndex=self.Settings.Items[args.Item:GetName()];
	if targetIndex~=nil then 
		if BackPack:GetItem(targetIndex)==nil then
			BackPack:PerformItemDrop(args.Item,targetIndex,Turbine.UI.Control.IsShiftKeyDown());
		else
			--so, target slot already occupied, now we have four options
			-- 1. do nothing :)
			-- 2. exchange the items (yugh!)
			-- 3. move the new item to the nearest free slot (ok, but can be messy)
			-- 4. move the new item to the nearest free slot, as long it is in the same bag (fair enough)
			--let's do option 4
			local targetBag=math.floor((targetIndex-1)/15);
			local targetBagFloor=targetBag*15+1;
			local targetBagCeil=targetBagFloor+14;
			local i=targetIndex;
			local j=targetIndex;
			local moved=false;
			while (i>=targetBagFloor or j<=targetBagCeil) and not moved do
				i=i-1;
				j=j+1;
				if i>=targetBagFloor and i>0 then
					if BackPack:GetItem(i)==nil then
						BackPack:PerformItemDrop(args.Item,i,Turbine.UI.Control.IsShiftKeyDown());
						args.NewIndex=i;
						moved=true;
					end
				end
				if j<=targetBagCeil and j<BackPack:GetSize() and not moved then
					if BackPack:GetItem(j)==nil then
						BackPack:PerformItemDrop(args.Item,j,Turbine.UI.Control.IsShiftKeyDown());
						args.NewIndex=j;
						moved=true;
					end
				end
			end
			--still no luck, and we 
			self:BackPackRefresh(args);
		end
	else
		self:BackPackRefresh(nil);
	end
end
--
function BagMemory:ItemAdded( sender, args )
	if Turbine.Engine.GetGameTime()-self.LastMoveTime<0.3 and args.Index==BackPack.LastMoveSlot then
		--this is a workaround: 
		--  if 2 items in the inventory are exchanged, 
		--  the item being dragged gets an ItemMoved event 
		--  while the item on the target slot gets an ItemAdded event
		self:BackPackRefresh(args);
	else
		--item picked up from somewhere, let's see if we can remember the last position and sort it somewhere
		self:BackPackMemory(args);
	end
end
function BagMemory:ItemMoved( sender, args )
	self.LastMoveTime=Turbine.Engine.GetGameTime();
	self:BackPackRefresh(args);
end
function BagMemory:SizeChanged( sender, args )
	self:BackPackRefresh();
end
function BagMemory:InitBackpack()
	self.LastMoveTime=Turbine.Engine.GetGameTime();
	BackPack.AddListener(self);
	self:BackPackRefresh();
end
function BagMemory:InitDefaults()
	self.Settings={
		Items={}
		};
	self.Defaults={
		Items={}
		};
	self.Resource={
		};
	self.BagStatusOld=nil;
end
function BagMemory:InitWindow()
	--no window needed yet
end
function BagMemory:LoadSettings()
	local fLoad = Turbine.PluginData.Load;
	local tSettings = fLoad( Turbine.DataScope.Character, "BagMemorySettings" );
	if ( type(tSettings) ~= "table" ) then
		self.Settings = PChaly.ChalyInv.DeepCopy( self.Defaults );
	else
		if ( Turbine.Engine.GetLocale() ~= "en" ) then
			tSettings = PChaly.ChalyInv.DeepCopyDecode( tSettings );
		end
		self.Settings = tSettings;
	end
end
function BagMemory:SaveSettings()
	local fSave = Turbine.PluginData.Save; 
	local tSettings = {};
	if ( Turbine.Engine.GetLocale() ~= "en" ) then
		tSettings = PChaly.ChalyInv.DeepCopyEncode( self.Settings );
	else
		tSettings = PChaly.ChalyInv.DeepCopy( self.Settings );
	end
	fSave( Turbine.DataScope.Character, "BagMemorySettings", tSettings );
end